#---
# Excerpted from "Rails for PHP Developers",
# published by The Pragmatic Bookshelf.
# Copyrights apply to this code. It may not be used to create training material, 
# courses, books, articles, and the like. Contact us if you are in doubt.
# We make no guarantees that this code is fit for any purpose. 
# Visit http://www.pragmaticprogrammer.com/titles/ndphpr for more book information.
#---
require File.dirname(__FILE__) + '/../test_helper'


class MeetingTest < Test::Unit::TestCase
  fixtures :meetings
  

  # CLASS METHODS ========================================

  def test_should_find_upcoming_meetings
    meetings = Meeting.upcoming_meetings
    assert meetings.size > 0

    for meeting in meetings
      assert meeting.meets_on > Time.now.to_date
    end
  end

  def test_should_find_past_meetings
    meetings = Meeting.past_meetings
    assert meetings.size > 0

    for meeting in meetings
      assert meeting.meets_on <= Time.now.to_date
    end
  end

  # INSTANCE METHODS ========================================

  
  def test_should_format_date_as_name
    meeting = meetings(:todays_meeting)
    assert_match /\w* \d{1,2}, \d{4}/i, meeting.name
  end
  

  # VALIDATION ========================================

  
  def test_should_create_meeting
    assert_difference 'Meeting.count', 1 do 
      meeting = create_meeting
      assert !meeting.new_record?
    end
  end

  def test_should_require_meets_on
    m = create_meeting(:meets_on => nil)
    assert m.errors.invalid?(:meets_on)
  end
  
  def test_should_have_a_location_with_four_char_minimum
    m = create_meeting(:location => 'boo')
    assert m.errors.invalid?(:location)
  end

  protected
    def create_meeting(options = {}) 
      attrs = { :meets_on    => "2008-01-01", 
                :location    => "University Library", 
                :description => "Lightning Talks" }.merge(options)
      Meeting.create(attrs)
    end
  
end
