#---
# Excerpted from "Rails for PHP Developers",
# published by The Pragmatic Bookshelf.
# Copyrights apply to this code. It may not be used to create training material, 
# courses, books, articles, and the like. Contact us if you are in doubt.
# We make no guarantees that this code is fit for any purpose. 
# Visit http://www.pragmaticprogrammer.com/titles/ndphpr for more book information.
#---
require File.dirname(__FILE__) + '/../test_helper'

class UserTest < Test::Unit::TestCase
  fixtures :users, :presentations

  # ASSOCIATIONS ========================================

  def test_should_have_many_presentations
    joe = users(:admin_user)

    assert joe.presentations.size > 0
    assert_kind_of Presentation, joe.presentations.first
  end

  # AUTHENTICATION ========================================

  def test_should_authenticate_user
    user = User.authenticate('joe@example.com', 'chunkybacon')
    assert_equal users(:admin_user), user
    
    # invalid username
    user = User.authenticate('not_a_user@example.com', 'chunkybacon')
    assert_not_equal users(:admin_user), user

    # invalid password
    user = User.authenticate('joe@example.com', 'invalid_pass')
    assert_not_equal users(:admin_user), user
  end

  def test_should_assign_new_password
    joe = users(:admin_user)
    joe.update_attributes(:password              => 'new pass', 
                          :password_confirmation => 'new pass')

    user = User.authenticate('joe@example.com', 'new pass')
    assert_equal users(:admin_user), user
  end

  def test_should_not_rehash_existing_password
    joe = users(:admin_user)
    joe.update_attributes(:email => 'joe2@example.com')

    user = User.authenticate('joe2@example.com', 'chunkybacon')
    assert_equal users(:admin_user), user
  end

  # VALIDATION ========================================

  def test_should_create_user
    assert_difference 'User.count', 1 do 
      user = create_user
      assert !user.new_record?
    end
  end

  def test_should_verify_email_and_password_length
    u = create_user(:email => "a")
    assert u.errors.invalid?(:email)

    u = create_user(:email => "test"*100)
    assert u.errors.invalid?(:email)    


    u = create_user(:password => "a")
    assert u.errors.invalid?(:password)

    u = create_user(:password => "test"*100)
    assert u.errors.invalid?(:password)    
  end

  def test_should_have_unique_emails
    u = create_user(:email => "joe@example.com")
    assert !u.valid?
    assert u.errors.invalid?(:email)
  end
  
  def test_should_verify_password_confirmation
    u = create_user(:password => "chunkybacon", 
                    :password_confirmation => "test")
    assert !u.valid?
    assert u.errors.invalid?(:password)    
  end

  def test_should_require_name_email_and_password
    u = create_user(:name => nil)
    assert u.errors.invalid?(:name)

    u = create_user(:email => nil)
    assert u.errors.invalid?(:email)

    u = create_user(:password => nil)
    assert u.errors.invalid?(:password)
  end

  protected
    def create_user(options = {})
      attrs = { :name                  => "Joe", 
                :email                 => "joe2@example.com", 
                :password              => "chunkybacon",
                :password_confirmation => "chunkybacon"}.merge(options)
      User.create(attrs)
    end
end
