#---
# Excerpted from "Rails for PHP Developers",
# published by The Pragmatic Bookshelf.
# Copyrights apply to this code. It may not be used to create training material, 
# courses, books, articles, and the like. Contact us if you are in doubt.
# We make no guarantees that this code is fit for any purpose. 
# Visit http://www.pragmaticprogrammer.com/titles/ndphpr for more book information.
#---

class UsersController < ApplicationController
  before_filter :admin_required, :only => [:destroy]
  before_filter :login_required, :only => [:edit, :update]


  before_filter :find_user, :except => [:index, :new, :create]
  before_filter :confirm_user_owns_record, :only => [:edit, :update]


  def index
    @users = User.find(:all, :order => "name")
  end

  def show
  end

  def new
    @user = User.new
  end

  def create
    @user = User.new(params[:user])

    if @user.save
      @current_user  = @user
      session[:user] = @user.id

      flash[:notice] = "Successfully Signed up"
      redirect_to :action => "show", :id => @user.id
    else
      render :action => "new"
    end
  end

  def edit
  end

  
  def update
    @user.admin = params[:user][:admin] if admin?
    if @user.update_attributes(params[:user])
      flash[:notice] = 'User successfully updated.'
      redirect_to :action => "show", :id => @user.id
    else
      render :action => "edit"
    end
  end
  

  def destroy
    @user.destroy
    flash[:notice] = 'User successfully destroyed.'
    redirect_to :action => "index"
  end

  private

  
  def find_user
    @user = User.find(params[:id])
  end

  # if the user is not an admin, make sure they can't edit other users
  def confirm_user_owns_record
    return if admin?
    if @user.id != @current_user.id
      redirect_to :action => "edit", :id => @current_user.id
    end
  end
  
end
