#---
# Excerpted from "Rails for PHP Developers",
# published by The Pragmatic Bookshelf.
# Copyrights apply to this code. It may not be used to create training material, 
# courses, books, articles, and the like. Contact us if you are in doubt.
# We make no guarantees that this code is fit for any purpose. 
# Visit http://www.pragmaticprogrammer.com/titles/ndphpr for more book information.
#---
class User < ActiveRecord::Base
  # unencrypted password
  attr_accessor :password

  # Protect method from mass-update
  attr_protected :admin

  # associations
  has_many :presentations

  # validation
  validates_length_of       :email, :within => 3..100
  validates_uniqueness_of   :email, :case_sensitive => false
  validates_presence_of     :name
  validates_length_of       :password, :within => 4..40, 
                                       :if => :password_required?
  validates_confirmation_of :password, :if => :password_required?

  # callbacks
  before_save :encrypt_password

  # encrypts given password using salt
  def self.encrypt(pass, salt)
    Digest::SHA1.hexdigest("--#{salt}--#{pass}--")
  end

  # authenticate by email/password 
  def self.authenticate(email, pass)
    user = find_by_email(email)
    user && user.authenticated?(pass) ? user : nil
  end

  # does the given password match the stored encrypted password
  def authenticated?(pass)
    encrypted_password == self.class.encrypt(pass, salt)
  end

  protected

  # no encrypted password yet OR password attribute is set
  def password_required?
    encrypted_password.blank? || !password.blank?
  end

  # before save - create salt, encrypt password
  def encrypt_password
    return if password.blank?
    if new_record?
      self.salt = Digest::SHA1.hexdigest("--#{Time.now}--#{email}--")
    end
    self.encrypted_password = self.class.encrypt(password, salt)
  end
end
