#---
# Excerpted from "Rails for PHP Developers",
# published by The Pragmatic Bookshelf.
# Copyrights apply to this code. It may not be used to create training material, 
# courses, books, articles, and the like. Contact us if you are in doubt.
# We make no guarantees that this code is fit for any purpose. 
# Visit http://www.pragmaticprogrammer.com/titles/ndphpr for more book information.
#---
require File.dirname(__FILE__) + '/../test_helper'

class MeetingsControllerTest < ActionController::TestCase
  fixtures :meetings, :presentations, :users

  def setup
    super
    @request.session[:user] = users(:admin_user).id
  end
  
  # authentication
  def test_should_require_admin
    [:new, :edit, :create, :update, :destroy].each do |action|
      post action, {}, :user => users(:non_admin_user).id
      assert_redirected_to :controller => "sessions"
    end
  end

  # index  
  def test_index_should_display_add_link_for_admin
    get :index, {}, :user => nil
    assert_select "p.add", false

    get :index, {}, :user => users(:admin_user).id
    assert_select "p.add"
  end

  # show
  def test_show_should_display_todays_meeting
    get :show, { :id => meetings(:todays_meeting) }
    assert_select "dd", /#{meetings(:todays_meeting).name}/
  end

  def test_show_should_display_associated_presentations
    get :show, { :id => meetings(:todays_meeting) }
    assert_select "#presentation_#{presentations(:rjs_templates).id}"
  end

  def test_show_should_display_add_and_modify_for_admin
    get :show, { :id => meetings(:todays_meeting) }, :user => nil
    assert_select "p.add",    false
    assert_select "p.modify", false

    get :show, { :id => meetings(:todays_meeting) }, 
                :user => users(:admin_user).id
    assert_select "p.add"
    assert_select "p.modify"
  end

  # create
  def test_create_should_validate_and_show_errors
    post :create, { :meeting => meeting_data(:location => "") }
    assert_select "#errorExplanation"
  end  

  def test_create_should_insert_meeting
    assert_difference "Meeting.count" do 
      post :create, { :meeting => meeting_data }
      assert_redirected_to :action => "index"
    end
  end

  # update
  def test_update_should_validate_and_show_errors
    post :update, { :id => meetings(:todays_meeting), 
                    :meeting => meeting_data(:location => "") }
    assert_select "#errorExplanation"
  end

  def test_update_should_update_meeting
    assert_difference "Meeting.count(:conditions => \"location = 'Ranch'\")" do 
      post :update, { :id => meetings(:todays_meeting), 
                      :meeting => meeting_data(:location => "Ranch") }
      assert_redirected_to :action => "show"    
    end
  end

  #destroy
  def test_destroy_should_delete_meeting
    assert_difference "Meeting.count", -1 do 
      post :destroy, { :id => meetings(:todays_meeting) }
      assert_redirected_to :action => "index"
    end
  end
  
  protected
    def meeting_data(options = {})
      { :meets_on    => "2007-01-01", 
        :location    => "Public Library", 
        :description => "Rails Hackfest" }.merge(options)
    end
end
