#---
# Excerpted from "Rails for PHP Developers",
# published by The Pragmatic Bookshelf.
# Copyrights apply to this code. It may not be used to create training material, 
# courses, books, articles, and the like. Contact us if you are in doubt.
# We make no guarantees that this code is fit for any purpose. 
# Visit http://www.pragmaticprogrammer.com/titles/ndphpr for more book information.
#---
require File.dirname(__FILE__) + '/../test_helper'

class PresentationsControllerTest < ActionController::TestCase
  fixtures :meetings, :presentations, :users

  def setup
    super
    @request.session[:user] = users(:admin_user).id
  end

  # authentication
  def test_should_require_admin
    [:new, :edit, :create, :update, :destroy].each do |action|
      post action, {}, :user => users(:non_admin_user).id
      assert_redirected_to :controller => "sessions"
    end
  end

  # create
  def test_create_should_validate_and_show_errors
    post :create, { :meeting_id => meetings(:todays_meeting), 
                    :presentation => presentation_data(:title => nil) }
    assert_select "#errorExplanation"
  end  

  def test_create_should_insert_meeting
    assert_difference "Presentation.count" do 
      post :create, { :meeting_id => meetings(:todays_meeting), 
                      :presentation => presentation_data }
      assert_redirected_to :controller => "meetings", :action => "show"
    end
  end

  # update
  def test_update_should_validate_and_show_errors
    post :update, { :meeting_id => meetings(:todays_meeting), 
                    :id => presentations(:rjs_templates), 
                    :presentation => presentation_data(:title => nil) }
    assert_select "#errorExplanation"
  end

  def test_update_should_update_meeting
    assert_difference "Presentation.count(:conditions => \"title = 'foo'\")" do 
      post :update, { :meeting_id => meetings(:todays_meeting), 
                      :id => presentations(:rjs_templates), 
                      :presentation => presentation_data(:title => 'foo') }
      assert_redirected_to :controller => "meetings", :action => "show"
    end
  end

  #destroy
  def test_destroy_should_delete_meeting
    assert_difference "Presentation.count", -1 do 
      post :destroy, { :meeting_id => meetings(:todays_meeting), 
                       :id => presentations(:rjs_templates) }
      assert_redirected_to :controller => "meetings", :action => "show"
    end
  end

  protected
    def presentation_data(options = {})
      { :title       => "RJS Templates", 
        :description => "Render Javascript Snippets", 
        :user        => users(:admin_user) }.merge(options)
    end
end
