#---
# Excerpted from "Rails for PHP Developers",
# published by The Pragmatic Bookshelf.
# Copyrights apply to this code. It may not be used to create training material, 
# courses, books, articles, and the like. Contact us if you are in doubt.
# We make no guarantees that this code is fit for any purpose. 
# Visit http://www.pragmaticprogrammer.com/titles/ndphpr for more book information.
#---
require File.dirname(__FILE__) + '/../test_helper'

class UsersControllerTest < ActionController::TestCase
  fixtures :meetings, :presentations, :users

  def setup
    super
    @request.session[:user] = users(:admin_user).id
  end

  # authentication
  def test_should_require_admin
    [:destroy].each do |action|
      post action, {}, :user => users(:non_admin_user).id
      assert_redirected_to :controller => "sessions"
    end
  end

  def test_should_authenticate
    [:edit, :update].each do |action|
      post action, {}, :user => nil
      assert_redirected_to :controller => "sessions"
    end
  end

  def test_should_confirm_user_owns_own_record
    [:edit, :update].each do |action|
      post action, { :id   => users(:admin_user) }, 
                   { :user => users(:non_admin_user).id }
      assert_redirected_to :action => "edit", :id => users(:non_admin_user).id
    end
  end

  def test_should_allow_admin_to_edit_any_record
    post :edit, { :id   => users(:non_admin_user) }, 
                { :user => users(:admin_user).id }
    assert_response :success

    post :update, { :id   => users(:non_admin_user), 
                    :user => user_data }, 
                  { :user => users(:admin_user).id }
    assert_redirected_to :action => "show"
  end

  # index  
  def test_index_should_display_admin_image
    get :index
    assert_select "img[alt=Admin]", 1
  end

  # show
  def test_show_should_display_current_user
    get :show, { :id => users(:admin_user) }
    assert_select ".email", users(:admin_user).email
  end

  def test_show_should_display_edit_for_own_record
    # anonymous
    get :show, { :id => users(:admin_user) }, :user => nil
    assert_select ".modify.user", false
    
    # not his record
    get :show, { :id => users(:admin_user) }, 
                 :user => users(:non_admin_user).id
    assert_select ".modify.user", false
    
    # his record
    get :show, { :id   => users(:non_admin_user) }, 
                 :user => users(:non_admin_user).id
    assert_select ".modify.user"
  end

  def test_show_should_display_edit_and_destroy_for_admin
    # anonymous
    get :show, { :id => users(:admin_user) }, :user => nil
    assert_select ".modify.admin", false
    
    # not admin
    get :show, { :id => users(:admin_user) }, 
                 :user => users(:non_admin_user).id
    assert_select ".modify.admin", false
    
    # admin
    get :show, { :id => users(:admin_user) }, 
                 :user => users(:admin_user).id
    assert_select ".modify.admin"
  end
  
  # edit
  def test_edit_should_allow_admins_to_change_admin_status
    # not admin
    get :edit, { :id => users(:admin_user) }, 
                 :user => users(:non_admin_user).id
    assert_select "#user_admin_true", false
      
    # admin
    get :edit, { :id => users(:admin_user) },   
                 :user => users(:admin_user).id
    assert_select "#user_admin_true"
  end

  # create
  def test_create_should_validate_and_show_errors
    post :create, { :user => user_data(:email => "") }
    assert_select "#errorExplanation"
  end  

  def test_create_should_insert_meeting
    assert_difference "User.count" do 
      post :create, { :user => user_data }
      assert_redirected_to :action => "show"
    end
  end

  # update
  def test_update_should_validate_and_show_errors
    post :update, { :id => users(:admin_user), 
                    :user => user_data(:email => "") }
    assert_select "#errorExplanation"
  end

  def test_update_should_update_meeting
    assert_difference "User.count(:conditions => \"name = 'foo'\")" do 
      post :update, { :id => users(:admin_user), 
                      :user => user_data(:name => "foo") }
      assert_redirected_to :action => "show"    
    end
  end

  #destroy
  def test_destroy_should_delete_meeting
    assert_difference "User.count", -1 do 
      post :destroy, { :id => users(:admin_user) }
      assert_redirected_to :action => "index"
    end
  end

  protected
    def user_data(options = {})
      { :name                  => "Joe", 
        :email                 => "joe2@example.com", 
        :password              => "chunkybacon",
        :password_confirmation => "chunkybacon"}.merge(options)
    end
end
