#---
# Excerpted from "Rails for PHP Developers",
# published by The Pragmatic Bookshelf.
# Copyrights apply to this code. It may not be used to create training material, 
# courses, books, articles, and the like. Contact us if you are in doubt.
# We make no guarantees that this code is fit for any purpose. 
# Visit http://www.pragmaticprogrammer.com/titles/ndphpr for more book information.
#---
require "#{File.dirname(__FILE__)}/../test_helper"

class UserStoriesTest < ActionController::IntegrationTest
  fixtures :users, :presentations, :meetings

  def test_should_create_new_user
    # home page
    get "/"
    assert_template "homepage/index"

    # new user page
    get "/signup"
    assert_template "users/new"

    # error attempting to create account
    post "/users/create", :user => user_data(:password => "foo")
    assert_template "users/new"
    assert_select "#errorExplanation"
    assert_nil session[:user]

    # created successfully
    post "/users/create", :user => user_data
    assert_redirected_to :controller => "users", :action => "show"
    assert_not_nil session[:user]
  end
  
  def test_should_login_user
    # home page
    get "/"
    assert_template "homepage/index"
    
    # login page
    get "/login"
    assert_template "sessions/new"
    
    # invalid login
    post "/sessions/create", login_data(:email => "bad")
    assert_template "sessions/new"
    assert_select "#flash_notice"
    assert_nil session[:user]

    # valid login
    post "/sessions/create", login_data
    assert_redirected_to :controller => "users", :action => "show"
    assert_not_nil session[:user]
  end
  
  def test_should_logout_user
    # valid login
    post "/sessions/create", login_data
    assert_redirected_to :controller => "users", :action => "show"
    assert_not_nil session[:user]

    # logout
    post '/logout'
    assert_redirected_to :controller => "sessions", :action => "new"
    assert_nil session[:user]
  end

  protected
    def user_data(options = {})
      { :name     => "Bob", 
        :email    => "bob@example.com", 
        :password => "chunkybacon", 
        :password_confirmation => "chunkybacon" }.merge(options)
    end
    
    def login_data(options = {}) 
      { :email    => "joe@example.com", 
        :password => "chunkybacon" }.merge(options)
    end
end
